#!/usr/bin/perl -w

# Copyright 2012 Kevin Ryde

# This file is part of Math-NumSeq.
#
# Math-NumSeq is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# Math-NumSeq is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with Math-NumSeq.  If not, see <http://www.gnu.org/licenses/>.

require 5;
use strict;
use List::Util 'min','max';

{
  require Math::NumSeq::SqrtContinuedPeriod;
  my $seq = Math::NumSeq::SqrtContinuedPeriod->new;
  my @periods;
  foreach my $i (2 .. 200) {
    push @periods, $seq->ith($i);
  }
  print "min ",min(@periods),"\n";
  print "max ",max(@periods),"\n";
  exit 0;
}


